﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using DG.Tweening;
using UnityEngine.UI;

public class LevelManager : MonoBehaviour 
{
	public static GameEventManager.GameState GAMESTATE;
	public static LevelTools.KillerList KILLER;
	public static GlobalTuning TUNING;

	public enum GameFeelList
	{
		None,
		Ascending,
		Descending
	};

	public GameFeelList Game;
	public Player Gamer;
	public LevelTools Tools;
	public tk2dCamera Cam;
	public CamTool Camt;
	public Color TintAdd;
	public Color TintMult;
	public List<MeshRenderer> Tiles;
	public List<Stars> StarList;
	public GameObject TPSpot;
	public List<Color> ColList;
	public Checkpoint currCp;
	public UIManager UIMan;
	public AudioSource Zik;
	public bool hasGrail;
	public int Timer;
	public int Respawns;


	private int _starct;  // the name field
	public int StarCount {    // the Name property
		get {
			return StarList.Count(s => s.Picked == true);
		}
		set {
			_starct =  StarList.Count(s => s.Picked == true);
		}
	}

	public float camSpeed;
	public float initCamSpeed;
	public float camSpeedMax;

	// Use this for initialization
	void Awake () 
	{
		TUNING = Instantiate(Resources.Load("Tuning")) as GlobalTuning;
		LevelManager.TUNING.Load<LevelManager>(this);
		Tools = gameObject.AddComponent<LevelTools>().Setup(this);
		Gamer = FindObjectOfType<Player>().Setup(this);
		Camt = FindObjectOfType<CamTool>().Setup(this);
		UIMan = FindObjectOfType<UIManager>().Setup(this);
		StarList = FindObjectsOfType<Stars> ().ToList ();
		StarList.ForEach ((Stars obj) => obj.Setup (this));
		DOTween.Init(false, true, LogBehaviour.Verbose);
		Cam = GetComponentInChildren<tk2dCamera>();
		TintMult = ColList [0];
		InvokeRepeating ("otherColor", 5f, 10f);
		Setup();
	}

	void Update()
	{
		if (GAMESTATE == GameEventManager.GameState.GameOver) {
			if (Input.GetKeyDown (KeyCode.R) || Input.GetKeyDown (KeyCode.R)) {
				GameEventManager.TriggerRespawn ("olol");
				Respawns++;
			}
		}
		Tiles.ForEach((MeshRenderer obj) => obj.sharedMaterial.SetColor("_MultColor", TintMult));
		Tiles.ForEach((MeshRenderer obj) => obj.sharedMaterial.SetColor("_AddCOlor", TintAdd));
		Cam.transform.position += new Vector3 (0f, camSpeed, 0f);
	}

	public void UpdateUI()
	{
		UIMan.starCounter.text = string.Format ("Stars : {0} / {1}", StarCount, StarList.Count);
	}

	public void otherColor()
	{
		DOTween.To(()=> TintMult, x=> TintMult = x, ColList.Next<Color>(TintMult),3f);
		// switch colors
	}
	private void addTimer()
	{
		Debug.Log (Timer);
		Timer++;
	}

	private void Setup()
	{
		GameEventManager.GameStart += GameStart;
		GameEventManager.GameOver += GameOver;
		GameEventManager.Respawn += Respawn;
		GameEventManager.EndGame += EndGame;
		GameEventManager.StartScroll += StartScroll;
	
		GameEventManager.TriggerGameStart("First Init LM");
	}

	public void MuteSound()
	{
		Zik.volume = (Zik.volume == 0f) ? 1f : 0f ;
	}

	IEnumerator RelaunchGame(float _time)
	{
		yield return new WaitForSeconds(_time);
		GAMESTATE = GameEventManager.GameState.Live;
		Player.State = Player.PlayerStateList.Alive;
	}

	public void Rewind()
	{
		DOTween.To(()=> TintMult, x=> TintMult = x, Color.white,1f).OnComplete(SwitchColor);
		DOTween.To(()=> camSpeed, x=> camSpeed = x, 0f,2f);
		StartCoroutine (WaitForSec (2f));
		CancelInvoke ("otherColor");
	}

	IEnumerator WaitForSec(float _time)
	{
		yield return new WaitForSeconds (_time);
		DOTween.To(()=> camSpeed, x=> camSpeed = x, -initCamSpeed,1f);
		initCamSpeed = -initCamSpeed;
		camSpeedMax = -camSpeedMax;
//		camSpeed = -initCamSpeed;
		
	}

	public void SwitchColor()
	{
		DOTween.To(()=> TintMult, x=> TintMult = x, Color.red,1f);
	}

	public void registerCP(Checkpoint _cp)
	{
		currCp = _cp;
		currCp.playPos = Gamer.transform.position;
		currCp.camPos = Cam.transform.position;	
		currCp.Timer = Timer;
		currCp.Sd.Play ();
		StarList.FindAll ((Stars obj) => obj.Picked == true).ForEach ((Stars obj) => obj.PickedSaved = true);
	}

	public void StartGame(Image _pan)
	{
		GameEventManager.TriggerRespawn ("LM");
		_pan.rectTransform.DOMoveY (-720f, 2f).SetEase (Ease.OutBounce);
	}
	public void Restart()
	{
		GameEventManager.TriggerRespawn ("LM");
	}

	public void OpenLDPage()
	{
		Application.OpenURL("http://ludumdare.com/compo/author/tavrox/");
	}

	private void GameStart()
	{
		if (this != null) {
			camSpeed = 0f;
		}
	}

	private void GameOver()
	{
		if (this != null)
		{
			CancelInvoke("addTimer");
			camSpeed = 0f;
		}
	}
	
	private void Respawn()
	{
		if (this != null)
		{
			InvokeRepeating ("addTimer",0f, 1f);
			if (Cam != null && currCp != null)
			{
				currCp.Sd.Play ();
				Cam.transform.position = new Vector3(Camt.initPos.x, currCp.transform.position.y, Camt.initPos.z);
				camSpeed = initCamSpeed;
				Timer = currCp.Timer;
				UpdateUI();
			}
		}
	}
	
	private void EndGame()
	{
		if (this != null)
		{
			CancelInvoke("addTimer");
			camSpeed = 0f;
		}
	}
	
	private void StartScroll()
	{
		if (this != null && Game == GameFeelList.None)
		{
			Game = GameFeelList.Ascending;
			Zik.Play();
			DOTween.To(()=> camSpeed, x=> camSpeed = x, initCamSpeed,2f);
		}
	}

	void OnDrawGizmos()
	{
		Tiles.ForEach((MeshRenderer obj) => obj.sharedMaterial.SetColor("_MultColor", TintMult));
		Tiles.ForEach((MeshRenderer obj) => obj.sharedMaterial.SetColor("_AddCOlor", TintAdd));
	}
}
