using UnityEngine;
using System.Collections;
using DG.Tweening;


[RequireComponent (typeof(BoxCollider))]
public class PlayerPhysics : MonoBehaviour {
	
	public LayerMask collisionMask;
	public LayerMask stickerMask;
	public LayerMask enemyMask;

	private BoxCollider collider;
	private Vector3 s;
	private Vector3 c;

	private Vector2 pushAmount;
	private Player Gamer;

	private Vector3 originalSize;
	private Vector3 originalCentre;
	private float colliderScale;
	
	private int collisionDivisionsX = 5;
	private int collisionDivisionsY = 4;
	private float deltaX = 0f;
	private float deltaY = 0f;

	public int blockedX;
	
	private float skin = .005f;
	
	[HideInInspector] public bool grounded;
	[HideInInspector] public bool movementStopped;
	[HideInInspector] public bool yMoveStopped;
	[HideInInspector] public bool xMoveStopped;


	Ray ray;
	RaycastHit hit;
	
	public PlayerPhysics Setup(Player _gamer) 
	{
		Gamer = _gamer;
		pushAmount.x = 0;
		pushAmount.y = 0;

		collider = GetComponent<BoxCollider>();
		colliderScale = transform.localScale.x;
		collisionMask = ~(((1 << collisionMask.value) | 1 << stickerMask.value));

		
		originalSize = collider.size;
		originalCentre = collider.center;
		SetCollider(originalSize,originalCentre);
		return this;
	}

	public void Move(Vector2 moveAmount) 
	{
		if (Player.State != Player.PlayerStateList.Cutscene && Player.State != Player.PlayerStateList.Dead)
		{
//			Debug.Log("move");
			blockedX = 0;
			deltaX = moveAmount.x;
			deltaY = moveAmount.y;
			Vector2 p = transform.position;
			
			// Check collisions above and below
			grounded = false;
			xMoveStopped = false;
			yMoveStopped = false;

			for (int i = 0; i<collisionDivisionsX; i ++) {
				float dir = Mathf.Sign(deltaY);
				float x = (p.x + c.x - s.x/2) + s.x/(collisionDivisionsX-1) * i; // Left, centre and then rightmost point of collider
				float y = p.y + c.y + s.y/2 * dir; // Bottom of collider
				
				ray = new Ray(new Vector2(x,y), new Vector2(0,dir));
				Debug.DrawRay(ray.origin,ray.direction, Color.red);
				
				if (Physics.Raycast(ray,out hit,Mathf.Abs(deltaY) + skin,enemyMask)) 
				{
					Debug.Log("Kill");
					GameEventManager.TriggerGameOver("Kill");
				}
				if (Physics.Raycast(ray,out hit,Mathf.Abs(deltaY) + skin,collisionMask)) 
				{
//					Debug.Log("vec");
					float dst = Vector3.Distance (ray.origin, hit.point);
					
					// Stop player's downwards movement after coming within skin width of a collider
					if (dst > skin) 
					{
//						Debug.Log("fall");
						deltaY = dst * dir - skin * dir;
					}
					else 
					{
//						Debug.Log("unfall");
						deltaY = 0;
					}

	//				 Check case ground or roof (Séverin)

					if (dir != 1) {
						grounded = true;
					} else {
						yMoveStopped = true;
					} 

					break;
					
				}
			}

			// Check collisions left and right
			movementStopped = false;
			for (int i = 0; i<collisionDivisionsY; i ++) {
				float dir = Mathf.Sign(deltaX);
				float x = p.x + c.x + s.x/2 * dir;
				float y = p.y + c.y - s.y/2 + s.y/(collisionDivisionsY-1) * i;




				ray = new Ray(new Vector2(x,y), new Vector2(dir,0));
				Debug.DrawRay(ray.origin,ray.direction, Color.red);
				if (Physics.Raycast(ray,out hit,Mathf.Abs(deltaX) + skin,enemyMask)) {
					Debug.Log("Kill");
					GameEventManager.TriggerGameOver("Kill");
				}
				
				if (Physics.Raycast(ray,out hit,Mathf.Abs(deltaX) + skin,collisionMask)) {

					float dst = Vector3.Distance (ray.origin, hit.point);
					if (dst > skin) {
						deltaX = dst * dir - skin * dir;
					}
					else {
						deltaX = 0;
					}
					
					movementStopped = true;
					break;
				}
			}

			if (!grounded && !movementStopped) {
				Vector3 playerDir = new Vector3(deltaX,deltaY);
				Vector3 o = new Vector3(p.x + c.x + s.x/2 * Mathf.Sign(deltaX),p.y + c.y + s.y/2 * Mathf.Sign(deltaY));
				ray = new Ray(o,playerDir.normalized); 
				
				if (Physics.Raycast(ray,Mathf.Sqrt(deltaX * deltaX + deltaY * deltaY),collisionMask)) 
				{
					grounded = true;
					deltaY = 0;
				}
			}

			if (grounded && (deltaX > 0 || deltaX < 0))
			{

			}

			deltaX = (Mathf.Abs(deltaX) < Gamer.PlayerMaxSpeed.x) ? deltaX : Mathf.Sign(deltaX) * Gamer.PlayerMaxSpeed.x;
			deltaY = (Mathf.Abs(deltaY) < Gamer.PlayerMaxSpeed.y) ? deltaY : Mathf.Sign(deltaY) * Gamer.PlayerMaxSpeed.y;

			Vector2 finalTransform = new Vector2(deltaX,deltaY);

			if (finalTransform != Vector2.zero)
			{

			}
			else
			{

			}

			if (grounded != true)
			{

			}
			transform.Translate(finalTransform,Space.World);
		}
	}

	// Set collider
	public void SetCollider(Vector3 size, Vector3 centre) {
		collider.size = size;
		collider.center = centre;
		
		s = size * colliderScale;
		c = centre * colliderScale;
	}
	
}
